@@ -75,8 +75,6 @@
     return fIsDragging;
 }
 
-
-
 @end
 
 #pragma mark Toolbar Identifiers
@@ -111,7 +109,8 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
             nil]];
 
         fJobGroups = [[NSMutableArray arrayWithCapacity:0] retain];
-       } 
+       }
+
         return self;
 }
 
@@ -120,14 +119,13 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     [fJobGroups setArray:QueueFileArray];
     fIsDragging = NO; 
     /* First stop any timer working now */
-    [self stopAnimatingCurrentJobGroupInQueue];
+    //[self stopAnimatingCurrentJobGroupInQueue];
     [fOutlineView reloadData];
     
     
     
     /* lets get the stats on the status of the queue array */
     
-    fEncodingQueueItem = 0;
     fPendingCount = 0;
     fCompletedCount = 0;
     fCanceledCount = 0;
@@ -140,11 +138,11 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
      * 2 == is yet to be encoded
      * 3 == cancelled
      */
-    
 	int i = 0;
+    NSDictionary *thisQueueDict = nil;
 	for(id tempObject in fJobGroups)
 	{
-		NSDictionary *thisQueueDict = tempObject;
+		thisQueueDict = tempObject;
 		if ([[thisQueueDict objectForKey:@"Status"] intValue] == 0) // Completed
 		{
 			fCompletedCount++;	
@@ -152,7 +150,11 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 		if ([[thisQueueDict objectForKey:@"Status"] intValue] == 1) // being encoded
 		{
 			fWorkingCount++;
-            fEncodingQueueItem = i;	
+            /* we have an encoding job so, lets start the animation timer */
+            if ([thisQueueDict objectForKey:@"EncodingPID"] && [[thisQueueDict objectForKey:@"EncodingPID"] intValue] == pidNum)
+            {
+                fEncodingQueueItem = i;
+            }
 		}
         if ([[thisQueueDict objectForKey:@"Status"] intValue] == 2) // pending		
         {
@@ -165,27 +167,24 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 		i++;
 	}
     
-    /* We should fire up the encoding timer here based on fWorkingCount */
-    
-    if (fWorkingCount > 0)
-    {
-        /* we have an encoding job so, lets start the animation timer */
-        [self startAnimatingCurrentWorkingEncodeInQueue];
-    }
-    
     /* Set the queue status field in the queue window */
     NSMutableString * string;
-    if (fPendingCount == 1)
+    if (fPendingCount == 0)
+    {
+        string = [NSMutableString stringWithFormat: NSLocalizedString( @"No encode pending", @"" )];
+    }
+    else if (fPendingCount == 1)
     {
         string = [NSMutableString stringWithFormat: NSLocalizedString( @"%d encode pending", @"" ), fPendingCount];
     }
     else
     {
-        string = [NSMutableString stringWithFormat: NSLocalizedString( @"%d encode(s) pending", @"" ), fPendingCount];
+        string = [NSMutableString stringWithFormat: NSLocalizedString( @"%d encodes pending", @"" ), fPendingCount];
     }
     [fQueueCountField setStringValue:string];
     
 }
+
 /* This method sets the status string in the queue window
  * and is called from Controller.mm (fHBController)
  * instead of running another timer here polling libhb
@@ -193,7 +192,9 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
  */
 - (void)setQueueStatusString: (NSString *)statusString
 {
-[fProgressTextField setStringValue:statusString];
+    
+    [fProgressTextField setStringValue:statusString];
+    
 }
 
 //------------------------------------------------------------------------------------
@@ -231,6 +232,12 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     fHBController = controller;
 }
 
+- (void)setPidNum: (int)myPidnum
+{
+    pidNum = myPidnum;
+    [fHBController writeToActivityLog: "HBQueueController : My Pidnum is %d", pidNum];
+}
+
 #pragma mark -
 
 //------------------------------------------------------------------------------------
@@ -240,6 +247,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 {
     [self showWindow:sender];
     [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"QueueWindowIsOpen"];
+    [self startAnimatingCurrentWorkingEncodeInQueue];
 }
 
 
@@ -254,7 +262,6 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     if( ![[self window] setFrameUsingName:@"Queue"] )
         [[self window] center];
     [self setWindowFrameAutosaveName:@"Queue"];
-    [[self window] setExcludedFromWindowsMenu:YES];
 
     /* lets setup our queue list outline view for drag and drop here */
     [fOutlineView registerForDraggedTypes: [NSArray arrayWithObject:DragDropSimplePboardType] ];
@@ -275,9 +282,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 
     // Show/hide UI elements
     fCurrentJobPaneShown = NO;     // it's shown in the nib
-    //[self showCurrentJobPane:NO];
 
-    //[self updateQueueCountField];
 }
 
 
@@ -287,6 +292,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
 - (void)windowWillClose:(NSNotification *)aNotification
 {
     [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"QueueWindowIsOpen"];
+    [self stopAnimatingCurrentJobGroupInQueue];
 }
 
 #pragma mark Toolbar
@@ -509,7 +515,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     }
     else
     { 
-    /* since we are not a currently encoding item, we can just be cancelled */
+    /* since we are not a currently encoding item, we can just be removed */
             [fHBController removeQueueFileItem:row];
     }
 }
@@ -522,19 +528,19 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
      * In this case, we are paused from the calling window, so calling
      * [fHBController Pause:NULL]; Again will resume encoding
      */
-       [fHBController Pause:NULL];
+    [fHBController Pause:NULL];
     if (returnCode == NSAlertOtherReturn)
     {
-    /* We need to save the currently encoding item number first */
-    int encodingItemToRemove = fEncodingQueueItem;
-    /* Since we are encoding, we need to let fHBController Cancel this job
-     * upon which it will move to the next one if there is one
-     */
-    [fHBController doCancelCurrentJob];
-    /* Now, we can go ahead and remove the job we just cancelled since
-     * we have its item number from above
-     */
-    [fHBController removeQueueFileItem:encodingItemToRemove];
+        /* We need to save the currently encoding item number first */
+        int encodingItemToRemove = fEncodingQueueItem;
+        /* Since we are encoding, we need to let fHBController Cancel this job
+         * upon which it will move to the next one if there is one
+         */
+        [fHBController doCancelCurrentJob];
+        /* Now, we can go ahead and remove the job we just cancelled since
+         * we have its item number from above
+         */
+        [fHBController removeQueueFileItem:encodingItemToRemove];
     }
     
 }
@@ -598,9 +604,53 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     }
 }
 
+
+//------------------------------------------------------------------------------------
+// Send the selected queue item back to the main window for rescan and possible edit.
+//------------------------------------------------------------------------------------
+- (IBAction)editSelectedQueueItem: (id)sender
+{
+    NSIndexSet * selectedRows = [fOutlineView selectedRowIndexes];
+    NSUInteger row = [selectedRows firstIndex];
+    if( row == NSNotFound )
+        return;
+    /* if this is a currently encoding job, we need to be sure to alert the user,
+     * to let them decide to cancel it first, then if they do, we can come back and
+     * remove it */
+    
+    if ([[[fJobGroups objectAtIndex:row] objectForKey:@"Status"] integerValue] == 1)
+    {
+       /* We pause the encode here so that it doesn't finish right after and then
+        * screw up the sync while the window is open
+        */
+       [fHBController Pause:NULL];
+         NSString * alertTitle = [NSString stringWithFormat:NSLocalizedString(@"Stop This Encode and Remove It ?", nil)];
+        // Which window to attach the sheet to?
+        NSWindow * docWindow = nil;
+        if ([sender respondsToSelector: @selector(window)])
+            docWindow = [sender window];
+        
+        
+        NSBeginCriticalAlertSheet(
+                                  alertTitle,
+                                  NSLocalizedString(@"Keep Encoding", nil),
+                                  nil,
+                                  NSLocalizedString(@"Stop Encoding and Delete", nil),
+                                  docWindow, self,
+                                  nil, @selector(didDimissCancelCurrentJob:returnCode:contextInfo:), nil,
+                                  NSLocalizedString(@"Your movie will be lost if you don't continue encoding.", nil));
+        
+    }
+    else
+    { 
+    /* since we are not a currently encoding item, we can just be cancelled */
+    [fHBController rescanQueueItemToMainWindow:[[fJobGroups objectAtIndex:row] objectForKey:@"SourcePath"] scanTitleNum:[[[fJobGroups objectAtIndex:row] objectForKey:@"TitleNumber"] integerValue] selectedQueueItem:row];
+    
+    }
+}
+
+
 #pragma mark -
-
-
 #pragma mark Animate Endcoding Item
 
 
@@ -634,7 +684,7 @@ static NSString*    HBQueuePauseResumeToolbarIdentifier       = @"HBQueuePauseRe
     }
 }
 
-
+/* We need to make sure we denote only working encodes even for multiple instances */
 - (void) animateWorkingEncodeIconInQueue
 {
     NSInteger row = fEncodingQueueItem; /// need to set to fEncodingQueueItem
@@ -852,21 +902,24 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         }
         
         /* check to see how many audio track lines to allow for */
-        if ([[queueItemToCheck objectForKey:@"Audio1Track"] intValue] > 0)
+		unsigned int ourMaximumNumberOfAudioTracks = [HBController maximumNumberOfAllowedAudioTracks];
+		int actualCountOfAudioTracks = 0;
+        BOOL autoPassthruPresent = NO;
+		for (unsigned int i = 1; i <= ourMaximumNumberOfAudioTracks; i++) {
+			if (0 < [[queueItemToCheck objectForKey: [NSString stringWithFormat: @"Audio%dTrack", i]] intValue])
+            {
+				actualCountOfAudioTracks++;
+			}
+            if (HB_ACODEC_AUTO_PASS == [[queueItemToCheck objectForKey: [NSString stringWithFormat: @"JobAudio%dEncoder", i]] intValue])
+            {
+                autoPassthruPresent = YES;
+            }
+		}
+		itemHeightForDisplay += (actualCountOfAudioTracks * rowHeightNonTitle);
+        
+        if (autoPassthruPresent == YES)
         {
-            itemHeightForDisplay +=  rowHeightNonTitle; 
-        }
-        if ([[queueItemToCheck objectForKey:@"Audio2Track"] intValue] > 0)
-        {
-            itemHeightForDisplay +=  rowHeightNonTitle; 
-        }
-        if ([[queueItemToCheck objectForKey:@"Audio3Track"] intValue] > 0)
-        {
-            itemHeightForDisplay +=  rowHeightNonTitle; 
-        }
-        if ([[queueItemToCheck objectForKey:@"Audio4Track"] intValue] > 0)
-        {
-            itemHeightForDisplay +=  rowHeightNonTitle; 
+            itemHeightForDisplay += rowHeightNonTitle * 2;
         }
         
         /* add in subtitle lines for each subtitle in the SubtitleList array */
@@ -917,16 +970,9 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
 
 - (id)outlineView:(NSOutlineView *)fOutlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
 {
-    // nb: The "desc" column is currently an HBImageAndTextCell. However, we are longer
-    // using the image portion of the cell so we could switch back to a regular NSTextFieldCell.
-    
     if ([[tableColumn identifier] isEqualToString:@"desc"])
     {
-        /* This should have caused the description we wanted to show*/
-        //return [item objectForKey:@"SourceName"];
         
-        /* code to build the description as per old queue */
-        //return [self formatEncodeItemDescription:item];
         
         /* Below should be put into a separate method but I am way too f'ing lazy right now */
         NSMutableAttributedString * finalString = [[[NSMutableAttributedString alloc] initWithString: @""] autorelease];
@@ -958,14 +1004,28 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
                                          nil];
         
         /* First line, we should strip the destination path and just show the file name and add the title num and chapters (if any) */
-        //finalDescription = [finalDescription stringByAppendingString:[NSString stringWithFormat:@"Source: %@ Output: %@\n", [item objectForKey:@"SourceName"],[item objectForKey:@"DestinationPath"]]];
         NSString * summaryInfo;
         
         NSString * titleString = [NSString stringWithFormat:@"Title %d", [[item objectForKey:@"TitleNumber"] intValue]];
         
-        NSString * chapterString = ([[item objectForKey:@"ChapterStart"] intValue] == [[item objectForKey:@"ChapterEnd"] intValue]) ?
-        [NSString stringWithFormat:@"Chapter %d", [[item objectForKey:@"ChapterStart"] intValue]] :
-        [NSString stringWithFormat:@"Chapters %d through %d", [[item objectForKey:@"ChapterStart"] intValue], [[item objectForKey:@"ChapterEnd"] intValue]];
+        NSString * startStopString = @"";
+        if ([[item objectForKey:@"fEncodeStartStop"] intValue] == 0)
+        {
+            /* Start Stop is chapters */
+            startStopString = ([[item objectForKey:@"ChapterStart"] intValue] == [[item objectForKey:@"ChapterEnd"] intValue]) ?
+            [NSString stringWithFormat:@"Chapter %d", [[item objectForKey:@"ChapterStart"] intValue]] :
+            [NSString stringWithFormat:@"Chapters %d through %d", [[item objectForKey:@"ChapterStart"] intValue], [[item objectForKey:@"ChapterEnd"] intValue]];
+        }
+        else if ([[item objectForKey:@"fEncodeStartStop"] intValue] == 1)
+        {
+            /* Start Stop is seconds */
+            startStopString = [NSString stringWithFormat:@"Seconds %d through %d", [[item objectForKey:@"StartSeconds"] intValue], [[item objectForKey:@"StartSeconds"] intValue] + [[item objectForKey:@"StopSeconds"] intValue]];
+        }
+        else if ([[item objectForKey:@"fEncodeStartStop"] intValue] == 2)
+        {
+            /* Start Stop is Frames */
+            startStopString = [NSString stringWithFormat:@"Frames %d through %d", [[item objectForKey:@"StartFrame"] intValue], [[item objectForKey:@"StartFrame"] intValue] + [[item objectForKey:@"StopFrame"] intValue]];
+        }
         
         NSString * passesString = @"";
         /* check to see if our first subtitle track is Foreign Language Search, in which case there is an in depth scan */
@@ -994,7 +1054,7 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         /* lets add the output file name to the title string here */
         NSString * outputFilenameString = [[item objectForKey:@"DestinationPath"] lastPathComponent];
         
-        summaryInfo = [NSString stringWithFormat: @" (%@, %@, %@) -> %@", titleString, chapterString, passesString, outputFilenameString];
+        summaryInfo = [NSString stringWithFormat: @" (%@, %@, %@) -> %@", titleString, startStopString, passesString, outputFilenameString];
         
         [finalString appendString:[NSString stringWithFormat:@"%@\n", summaryInfo] withAttributes:detailAttr];  
         
@@ -1007,91 +1067,41 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         [finalString appendString:[NSString stringWithFormat:@"%@\n", [item objectForKey:@"PresetName"]] withAttributes:detailAttr];
         
         /* Third Line  (Format Summary) */
-        NSString * audioCodecSummary = @"";
+        NSString * audioCodecSummary = @"";	//	This seems to be set by the last track we have available...
         /* Lets also get our audio track detail since we are going through the logic for use later */
-        NSString * audioDetail1 = @"";
-        NSString * audioDetail2 = @"";
-        NSString * audioDetail3 = @"";
-        NSString * audioDetail4 = @"";
-        if ([[item objectForKey:@"Audio1Track"] intValue] > 0)
-        {
-            audioCodecSummary = [NSString stringWithFormat:@"%@", [item objectForKey:@"Audio1Encoder"]];
-            audioDetail1 = [NSString stringWithFormat:@"%@ Encoder: %@ Mixdown: %@ SampleRate: %@(khz) Bitrate: %@(kbps)",
-                            [item objectForKey:@"Audio1TrackDescription"] ,
-                            [item objectForKey:@"Audio1Encoder"],
-                            [item objectForKey:@"Audio1Mixdown"] ,
-                            [item objectForKey:@"Audio1Samplerate"],
-                            [item objectForKey:@"Audio1Bitrate"]];
-            
-            if ([[item objectForKey:@"Audio1TrackDRCSlider"] floatValue] > 0.00)
+		unsigned int ourMaximumNumberOfAudioTracks = [HBController maximumNumberOfAllowedAudioTracks];
+		NSMutableArray *audioDetails = [NSMutableArray arrayWithCapacity: ourMaximumNumberOfAudioTracks];
+		NSString *base;
+		NSString *detailString;
+		NSNumber *drc;
+        NSNumber *gain;
+        BOOL autoPassthruPresent = NO;
+		for (unsigned int i = 1; i <= ourMaximumNumberOfAudioTracks; i++) {
+			base = [NSString stringWithFormat: @"Audio%d", i];
+			if (0 < [[item objectForKey: [base stringByAppendingString: @"Track"]] intValue])
             {
-                audioDetail1 = [NSString stringWithFormat:@"%@, DRC: %@",audioDetail1,[item objectForKey:@"Audio1TrackDRCSlider"]];
-            }
-            else
-            {
-                audioDetail1 = [NSString stringWithFormat:@"%@, DRC: Off",audioDetail1];
-            }
-        }
+				audioCodecSummary = [NSString stringWithFormat: @"%@", [item objectForKey: [base stringByAppendingString: @"Encoder"]]];
+				drc = [item objectForKey: [base stringByAppendingString: @"TrackDRCSlider"]];
+                gain = [item objectForKey: [base stringByAppendingString: @"TrackGainSlider"]];
+				detailString = [NSString stringWithFormat: @"%@ Encoder: %@ Mixdown: %@ SampleRate: %@(khz) Bitrate: %@(kbps), DRC: %@, Gain: %@",
+								[item objectForKey: [base stringByAppendingString: @"TrackDescription"]],
+								[item objectForKey: [base stringByAppendingString: @"Encoder"]],
+								[item objectForKey: [base stringByAppendingString: @"Mixdown"]],
+								[item objectForKey: [base stringByAppendingString: @"Samplerate"]],
+								[item objectForKey: [base stringByAppendingString: @"Bitrate"]],
+                                (0.0 < [drc floatValue]) ? (NSObject *)drc : (NSObject *)@"Off",
+								(0.0 != [gain floatValue]) ? (NSObject *)gain : (NSObject *)@"Off"
+								]
+                                ;
+				[audioDetails addObject: detailString];
+                // check if we have an Auto Passthru output track
+                if ([[item objectForKey: [NSString stringWithFormat: @"Audio%dEncoder", i]] isEqualToString: @"Auto Passthru"])
+                {
+                    autoPassthruPresent = YES;
+                }
+			}
+		}
         
-        if ([[item objectForKey:@"Audio2Track"] intValue] > 0)
-        {
-            audioCodecSummary = [NSString stringWithFormat:@"%@, %@",audioCodecSummary ,[item objectForKey:@"Audio2Encoder"]];
-            audioDetail2 = [NSString stringWithFormat:@"%@ Encoder: %@ Mixdown: %@ SampleRate: %@(khz) Bitrate: %@(kbps)",
-                            [item objectForKey:@"Audio2TrackDescription"] ,
-                            [item objectForKey:@"Audio2Encoder"],
-                            [item objectForKey:@"Audio2Mixdown"] ,
-                            [item objectForKey:@"Audio2Samplerate"],
-                            [item objectForKey:@"Audio2Bitrate"]];
-            
-            if ([[item objectForKey:@"Audio2TrackDRCSlider"] floatValue] > 0.00)
-            {
-                audioDetail2 = [NSString stringWithFormat:@"%@, DRC: %@",audioDetail2,[item objectForKey:@"Audio2TrackDRCSlider"]];
-            }
-            else
-            {
-                audioDetail2 = [NSString stringWithFormat:@"%@, DRC: Off",audioDetail2];
-            }
-        }
-        
-        if ([[item objectForKey:@"Audio3Track"] intValue] > 0)
-        {
-            audioCodecSummary = [NSString stringWithFormat:@"%@, %@",audioCodecSummary ,[item objectForKey:@"Audio3Encoder"]];
-            audioDetail3 = [NSString stringWithFormat:@"%@ Encoder: %@ Mixdown: %@ SampleRate: %@(khz) Bitrate: %@(kbps)",
-                            [item objectForKey:@"Audio3TrackDescription"] ,
-                            [item objectForKey:@"Audio3Encoder"],
-                            [item objectForKey:@"Audio3Mixdown"] ,
-                            [item objectForKey:@"Audio3Samplerate"],
-                            [item objectForKey:@"Audio3Bitrate"]];
-            
-            if ([[item objectForKey:@"Audio3TrackDRCSlider"] floatValue] > 0.00)
-            {
-                audioDetail3 = [NSString stringWithFormat:@"%@, DRC: %@",audioDetail3,[item objectForKey:@"Audio3TrackDRCSlider"]];
-            }
-            else
-            {
-                audioDetail3 = [NSString stringWithFormat:@"%@, DRC: Off",audioDetail3];
-            }
-        }
-        
-        if ([[item objectForKey:@"Audio4Track"] intValue] > 0)
-        {
-            audioCodecSummary = [NSString stringWithFormat:@"%@, %@",audioCodecSummary ,[item objectForKey:@"Audio3Encoder"]];
-            audioDetail4 = [NSString stringWithFormat:@"%@ Encoder: %@ Mixdown: %@ SampleRate: %@(khz) Bitrate: %@(kbps)",
-                            [item objectForKey:@"Audio4TrackDescription"] ,
-                            [item objectForKey:@"Audio4Encoder"],
-                            [item objectForKey:@"Audio4Mixdown"] ,
-                            [item objectForKey:@"Audio4Samplerate"],
-                            [item objectForKey:@"Audio4Bitrate"]];
-            
-            if ([[item objectForKey:@"Audio4TrackDRCSlider"] floatValue] > 0.00)
-            {
-                audioDetail4 = [NSString stringWithFormat:@"%@, DRC: %@",audioDetail4,[item objectForKey:@"Audio4TrackDRCSlider"]];
-            }
-            else
-            {
-                audioDetail4 = [NSString stringWithFormat:@"%@, DRC: Off",audioDetail4];
-            }
-        }
         
         NSString * jobFormatInfo;
         if ([[item objectForKey:@"ChapterMarkers"] intValue] == 1)
@@ -1136,6 +1146,7 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         [finalString appendString: @"Destination: " withAttributes:detailBoldAttr];
         [finalString appendString: [item objectForKey:@"DestinationPath"] withAttributes:detailAttr];
         [finalString appendString:@"\n" withAttributes:detailAttr];
+        
         /* Fifth Line Picture Details*/
         NSString * pictureInfo;
         pictureInfo = [NSString stringWithFormat:@"%@", [item objectForKey:@"PictureSizingSummary"]];
@@ -1143,6 +1154,7 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         {
             pictureInfo = [pictureInfo stringByAppendingString:@" Keep Aspect Ratio"];
         }
+        
         if ([[item objectForKey:@"VideoGrayScale"] intValue] == 1)
         {
             pictureInfo = [pictureInfo stringByAppendingString:@", Grayscale"];
@@ -1172,7 +1184,6 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         {
             if ([[item objectForKey:@"PictureDecomb"] intValue] != 0)
             {
-                pictureFiltersPresent = YES;
                 if( [[item objectForKey:@"PictureDecomb"] intValue] == 1)
                 {
                     pictureFiltersPresent = YES;
@@ -1183,6 +1194,11 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
                     pictureFiltersPresent = YES;
                     pictureFilters = [pictureFilters stringByAppendingString:@" - Decomb (Default)"];
                 }
+                else if( [[item objectForKey:@"PictureDecomb"] intValue] == 3)
+                {
+                    pictureFiltersPresent = YES;
+                    pictureFilters = [pictureFilters stringByAppendingString:@" - Decomb (Fast)"];
+                }
             }
         }
         else
@@ -1256,21 +1272,28 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         /* for framerate look to see if we are using vfr detelecine */
         if ([[item objectForKey:@"JobIndexVideoFramerate"] intValue] == 0)
         {
-            if ([[item objectForKey:@"PictureDetelecine"] intValue] == 1)
+            if ([[item objectForKey:@"VideoFramerateMode"] isEqualToString:@"vfr"])
             {
                 /* we are using same as source with vfr detelecine */
-                videoInfo = [NSString stringWithFormat:@"%@ Framerate: Same as source (vfr detelecine)", videoInfo];
+                videoInfo = [NSString stringWithFormat:@"%@ Framerate: Same as source (Variable Frame Rate)", videoInfo];
             }
             else
             {
                 /* we are using a variable framerate without dropping frames */
-                videoInfo = [NSString stringWithFormat:@"%@ Framerate: Same as source (variable)", videoInfo];
+                videoInfo = [NSString stringWithFormat:@"%@ Framerate: Same as source (Constant Frame Rate)", videoInfo];
             }
         }
         else
         {
             /* we have a specified, constant framerate */
-            videoInfo = [NSString stringWithFormat:@"%@ Framerate: %@ (constant framerate)", videoInfo ,[item objectForKey:@"VideoFramerate"]];
+            if ([[item objectForKey:@"VideoFramerateMode"] isEqualToString:@"pfr"])
+            {
+            videoInfo = [NSString stringWithFormat:@"%@ Framerate: %@ (Peak Frame Rate)", videoInfo ,[item objectForKey:@"VideoFramerate"]];
+            }
+            else
+            {
+            videoInfo = [NSString stringWithFormat:@"%@ Framerate: %@ (Constant Frame Rate)", videoInfo ,[item objectForKey:@"VideoFramerate"]];
+            }
         }
         
         if ([[item objectForKey:@"VideoQualityType"] intValue] == 0)// Target Size MB
@@ -1293,41 +1316,105 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         if ([[item objectForKey:@"VideoEncoder"] isEqualToString: @"H.264 (x264)"])
         {
             [finalString appendString: @"x264 Options: " withAttributes:detailBoldAttr];
-            [finalString appendString: [item objectForKey:@"x264Option"] withAttributes:detailAttr];
+            if ([item objectForKey:@"x264Option"])
+            {
+                [finalString appendString: [item objectForKey:@"x264Option"] withAttributes:detailAttr];
+            }
+            else
+            {
+                [finalString appendString: @"x264 defaults" withAttributes:detailAttr];   
+            }
             [finalString appendString:@"\n" withAttributes:detailAttr];
         }
+        /*If we are not x264 and we are not Theora then we must be FFmpeg (lavc) */
+        else if (![[item objectForKey:@"VideoEncoder"] isEqualToString: @"VP3 (Theora)"])
+        {
+            [finalString appendString: @"FFmpeg (lavc) Options: " withAttributes:detailBoldAttr];
+            if ([item objectForKey:@"lavcOption"])
+            {
+                [finalString appendString: [item objectForKey:@"lavcOption"] withAttributes:detailAttr];
+            }
+            else
+            {
+                [finalString appendString: @"FFmpeg (lavc) defaults" withAttributes:detailAttr];   
+            }
+            [finalString appendString:@"\n" withAttributes:detailAttr];
+        }
+        
         
         
         
         /* Seventh Line Audio Details*/
-        if ([audioDetail1 length] != 0)
+        NSEnumerator *audioDetailEnumerator = [audioDetails objectEnumerator];
+		NSString *anAudioDetail;
+		int audioDetailCount = 0;
+		while (nil != (anAudioDetail = [audioDetailEnumerator nextObject])) {
+			audioDetailCount++;
+			if (0 < [anAudioDetail length]) {
+				[finalString appendString: [NSString stringWithFormat: @"Audio Track %d ", audioDetailCount] withAttributes: detailBoldAttr];
+				[finalString appendString: anAudioDetail withAttributes: detailAttr];
+				[finalString appendString: @"\n" withAttributes: detailAttr];
+			}
+		}
+        
+        /* Eigth Line Auto Passthru Details */
+        // only print Auto Passthru settings if we have an Auro Passthru output track
+        if (autoPassthruPresent == YES)
         {
-            [finalString appendString: @"Audio Track 1: " withAttributes:detailBoldAttr];
-            [finalString appendString: audioDetail1 withAttributes:detailAttr];
-            [finalString appendString:@"\n" withAttributes:detailAttr];
+            NSString *autoPassthruFallback = @"", *autoPassthruCodecs = @"";
+            autoPassthruFallback = [autoPassthruFallback stringByAppendingString: [item objectForKey: @"AudioEncoderFallback"]];
+            if (0 < [[item objectForKey: @"AudioAllowAACPass"] intValue])
+            {
+                autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @"AAC"];
+            }
+            if (0 < [[item objectForKey: @"AudioAllowAC3Pass"] intValue])
+            {
+                if (0 < [autoPassthruCodecs length])
+                {
+                    autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @", "];
+                }
+                autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @"AC3"];
+            }
+            if (0 < [[item objectForKey: @"AudioAllowDTSHDPass"] intValue])
+            {
+                if (0 < [autoPassthruCodecs length])
+                {
+                    autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @", "];
+                }
+                autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @"DTS-HD"];
+            }
+            if (0 < [[item objectForKey: @"AudioAllowDTSPass"] intValue])
+            {
+                if (0 < [autoPassthruCodecs length])
+                {
+                    autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @", "];
+                }
+                autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @"DTS"];
+            }
+            if (0 < [[item objectForKey: @"AudioAllowMP3Pass"] intValue])
+            {
+                if (0 < [autoPassthruCodecs length])
+                {
+                    autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @", "];
+                }
+                autoPassthruCodecs = [autoPassthruCodecs stringByAppendingString: @"MP3"];
+            }
+            [finalString appendString: @"Auto Passthru Codecs: " withAttributes: detailBoldAttr];
+            if (0 < [autoPassthruCodecs length])
+            {
+                [finalString appendString: autoPassthruCodecs withAttributes: detailAttr];
+            }
+            else
+            {
+                [finalString appendString: @"None" withAttributes: detailAttr];
+            }
+            [finalString appendString: @"\n" withAttributes: detailAttr];
+            [finalString appendString: @"Auto Passthru Fallback: " withAttributes: detailBoldAttr];
+            [finalString appendString: autoPassthruFallback withAttributes: detailAttr];
+            [finalString appendString: @"\n" withAttributes: detailAttr];
         }
         
-        if ([audioDetail2 length] != 0)
-        {
-            [finalString appendString: @"Audio Track 2: " withAttributes:detailBoldAttr];
-            [finalString appendString: audioDetail2 withAttributes:detailAttr];
-            [finalString appendString:@"\n" withAttributes:detailAttr];
-        }
-        
-        if ([audioDetail3 length] != 0)
-        {
-            [finalString appendString: @"Audio Track 3: " withAttributes:detailBoldAttr];
-            [finalString appendString: audioDetail3 withAttributes:detailAttr];
-            [finalString appendString:@"\n" withAttributes:detailAttr];
-        }
-        
-        if ([audioDetail4 length] != 0)
-        {
-            [finalString appendString: @"Audio Track 4: " withAttributes:detailBoldAttr];
-            [finalString appendString: audioDetail4 withAttributes:detailAttr];
-            [finalString appendString:@"\n" withAttributes:detailAttr];
-        }
-        /* Eighth Line Subtitle Details */
+        /* Ninth Line Subtitle Details */
         
         int i = 0;
         NSEnumerator *enumerator = [[item objectForKey:@"SubtitleList"] objectEnumerator];
@@ -1386,7 +1473,7 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         return @"";
     }
 }
-
+/* This method inserts the proper action icons into the far right of the queue window */
 - (void)outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
 {
     if ([[tableColumn identifier] isEqualToString:@"desc"])
@@ -1403,7 +1490,8 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
     {
         [cell setEnabled: YES];
         BOOL highlighted = [outlineView isRowSelected:[outlineView rowForItem: item]] && [[outlineView window] isKeyWindow] && ([[outlineView window] firstResponder] == outlineView);
-        if ([[item objectForKey:@"Status"] intValue] == 0)
+        
+        if ([[item objectForKey:@"Status"] intValue] == 0 || ([[item objectForKey:@"Status"] intValue] == 1 && [[item objectForKey:@"EncodingPID"] intValue] != pidNum))
         {
             [cell setAction: @selector(revealSelectedQueueItem:)];
             if (highlighted)
@@ -1416,21 +1504,23 @@ return ![(HBQueueOutlineView*)outlineView isDragging];
         }
         else
         {
-            [cell setAction: @selector(removeSelectedQueueItem:)];
-            if (highlighted)
-            {
-                [cell setImage:[NSImage imageNamed:@"DeleteHighlight"]];
-                [cell setAlternateImage:[NSImage imageNamed:@"DeleteHighlightPressed"]];
-            }
-            else
-                [cell setImage:[NSImage imageNamed:@"Delete"]];
+            
+                [cell setAction: @selector(removeSelectedQueueItem:)];
+                if (highlighted)
+                {
+                    [cell setImage:[NSImage imageNamed:@"DeleteHighlight"]];
+                    [cell setAlternateImage:[NSImage imageNamed:@"DeleteHighlightPressed"]];
+                }
+                else
+                    [cell setImage:[NSImage imageNamed:@"Delete"]];
+   
         }
     }
 }
 
 - (void)outlineView:(NSOutlineView *)outlineView willDisplayOutlineCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
 {
-    // By default, the discolsure image gets centered vertically in the cell. We want
+    // By default, the disclosure image gets centered vertically in the cell. We want
     // always at the top.
     if ([outlineView isItemExpanded: item])
         [cell setImagePosition: NSImageAbove];
